# -*- coding: utf-8 -*-

import Tkinter
import types
from UnicodeHandler import fsdecode, fsencode

class BaseDialog(Tkinter.Toplevel):
    def __init__(self, *args, **kw):
        Tkinter.Toplevel.__init__(self, *args, **kw)
        self.withdraw()
        self.title('Update Notifier Applet')
        self.protocol('WM_DELETE_WINDOW', self.hide)
        self._hotkeys = []

    def assign_hotkey(self, button):
        i = 0
        letters = 'abcdefghijklmnopqrstuvwxyz'
        for char in button['text']:
            # if we call fsencode for the text string as a whole, counting
            # might get out of sync if we have a two-byte character
            # like a german umlaut in the button text
            c = fsencode(char).lower()
            if len(c) == 1 and c in letters and ord(c) < 128 and not c in self._hotkeys:
                self._hotkeys.append(c)
                button.configure(underline=i)
                seq, cmd = '<Alt-%s>' % c, button['command']
                # button['command'] is a str, so no need to bother with events
                self.bind(seq, cmd)
                break
            i += 1
        return seq

    def centeronscreen(self):
        parent = self.winfo_parent()
        if type(parent) == types.StringType:
            parent = self._nametowidget(parent)

        # Find size of window.
        self.update_idletasks()
        width = self.winfo_width()
        height = self.winfo_height()
        if width == 1 and height == 1:
            # If the window has not yet been displayed, its size is
            # reported as 1x1, so use requested size.
            width = self.winfo_reqwidth()
            height = self.winfo_reqheight()

        # Place in centre of screen:
        x = (self.winfo_screenwidth() - width) / 2 - parent.winfo_vrootx()
        y = (self.winfo_screenheight() - height) / 2 - parent.winfo_vrooty()
        if x < 0:
            x = 0
        if y < 0:
            y = 0
        return '+%d+%d' % (x, y)

    def hide(self):
        self.withdraw()

    def show(self):
        if not self.winfo_ismapped():
            self.geometry(self.centeronscreen())
            self.deiconify()
            self.wait_visibility()
        self.lift()
