# -*- coding: utf-8 -*-

import os
from UnicodeHandler import fsencode, fsdecode
import fileinput

class BaseConfigHandler:
    def __init__(self):
        self._opts = {}
        self._int_opts = []

    def _get_key_and_value(self, line):
        try:
            l = line.split('=')
            key = l[0].strip()
            value = l[1].strip()
            return (key, value)
        except:
            return ('', '')

    def _load(self):
        try:
            for line in fileinput.input(self._conffile):
                line = line.strip()
                key, value = self._get_key_and_value(line)
                if self._opts.has_key(key):
                    if key in self._int_opts:
                        try:
                            value = int(value)
                        except ValueError:
                            pass
                    self.set(key, value)
        except (IOError, OSError):
            print('Unable to open configuration file "%s".' % self._conffile)

    def get(self, opt=None):
        if opt:
            return self._opts[opt]
        else:
            # query the state of all options
            opts = self._opts.copy()
            return opts

    def set(self, opt, value):
        ok = True
        if opt in self._int_opts:
            try:
                value = int(value)
            except ValueError:
                ok = False
        else:
            value = str(fsencode(value))
        if ok:
            self._opts[opt] = value
        return ok

    def save(self):
        out = ''
        for key in self._opts:
            value = str(fsencode(self._opts[key]))
            out += '%s=%s\n' %(key, value)
        try:
            f = open(self._conffile, 'w')
            f.write(out)
            f.close()
            return 1
        except (IOError, OSError):
            print('Unable to write configuration file "%s".' % self._conffile)
            return 0

###############################################################################

class SysConfigHandler(BaseConfigHandler):
    def __init__(self):
        BaseConfigHandler.__init__(self)
        confdir = os.path.join('/etc', 'un-applet')
        self._conffile = os.path.join(confdir, 'un-applet.conf')
        if not os.path.isdir(confdir):
            try:
                os.mkdir(confdir)
            except (IOError, OSError):
                print('un-applet-system-settings: Error: unable to create directory: "%s".' % confdir)

        self._opts = {'DIST_UPGRADE': '"1"',
                      'NEED_APPLET': '"1"',
                      'NOTIFY_HOLDS': '"0"',
                      'NOTIFY_NEW': '"0"',
                      'UPDATE_INTERVAL': '10500',
                      'PM_PROGRAM': '"apt-get"',
                      'VERIFY_MIRRORS': '"1"'
                      }
        self._int_opts = ('UPDATE_INTERVAL')
        if not os.path.isfile(self._conffile):
            self.save()
        self._load()

    def permissions(self):
        e = os.path.isfile(self._conffile)
        if e:
            r = os.access(self._conffile, os.R_OK)
            w = os.access(self._conffile, os.W_OK)
        else:
            r = os.access(os.path.dirname(self._conffile), os.R_OK)
            w = os.access(os.path.dirname(self._conffile), os.W_OK)
        return self._conffile, e, r, w

###############################################################################

class UserConfigHandler(BaseConfigHandler):
    def __init__(self):
        BaseConfigHandler.__init__(self)
        self._confdir = os.path.join(fsencode(os.getenv('HOME')), '.config','un-applet')
        self._conffile = os.path.join(self._confdir, 'un-applet.conf')
        if not os.path.isdir(self._confdir):
            try:
                os.mkdir(self._confdir)
            except (IOError, OSError):
                print('Error: unable to create directory: "%s".' % self._confdir)

        self._opts = {'TERM_WIDTH': 80,
                      'TERM_HEIGHT': 24,
                      'TERM_XOFF': 100,
                      'TERM_YOFF': 100,
                      'TERM_FG': 'white',
                      'TERM_BG': 'darkblue',
                      'GKSU_CMD': 'gksu',
                      'POLL_INTERVAL': 10,
                      'PREF_TERM': 'mlterm',
                      'STARTUP_DELAY': 2,
                      'ICON_THEME': 'Default'
                      }
        self._int_opts = ('TERM_WIDTH', 'TERM_HEIGHT', 'TERM_XOFF',
                          'TERM_YOFF', 'POLL_INTERVAL', 'STARTUP_DELAY')
        if not os.path.isfile(self._conffile):
            self.save()
        self._load()

###############################################################################

# convenience stuff for user config

_handler = None
def initialize():
    global _handler, set, get, save
    if _handler is None:
        _handler = UserConfigHandler()
        set = _handler.set
        get = _handler.get
        save = _handler.save
