# -*- coding: utf-8 -*-

import Tkinter

class Counter(Tkinter.Frame):
    def __init__(self, master, label=None, label2=None, from_=0, to=100,
                 increment=1, value=0, datatype='integer', digits=0,
                 width=5, command=None, **kw):
        Tkinter.Frame.__init__(self, master)

        self._command = command
        self._var = Tkinter.StringVar()
        self.label, self.label2 = None, None
        if label:
            self.label = Tkinter.Label(self, text=label)
            self.label.pack(side='left', padx=(0, 5))
        self.spinbox = Tkinter.Spinbox(self, from_=from_, to=to,
                                       increment=increment,
                                       textvariable=self._var, validate='all',
                                       invcmd=self.bell, width=width, **kw)
        self.spinbox.pack(side='left', fill='x', expand=1)
        if label2:
            self.label2 = Tkinter.Label(self, text=label2)
            self.label2.pack(side='left', padx=(5, 0))
        if digits:
            format = '%' + '0.%df' % digits
            self.spinbox.configure(format=format)

        self._datatype = datatype
        self._oldvalue = value
        self.set(value)

        vcmd = (self.register(self._validate), '%s', '%P')
        self.spinbox.configure(vcmd=vcmd)

    def _validate(self, old, new):
        if (old != '') and (float(self.spinbox['from']) <= float(old) <= float(self.spinbox['to'])):
            self._oldvalue = old
        if new == '':
            return 1
        try:
            if self._datatype == 'integer':
                x = int(new)
            elif self._datatype == 'float':
                x = float(new)
            if 0 <= x <= float(self.spinbox['to']):
                if self._command and (new != self._oldvalue):
                    self._command(x)
                return 1
            return 0
        except ValueError:
            return 0

    def get(self):
        val = self.spinbox.get()
        if val == '':
            self.set(self._oldvalue)
        # now see if the current value is smaller than the allowed minimum
        # this is allowed for editing but not for retrieving the value
        elif self._datatype == 'integer':
            if int(val) < float(self.spinbox['from']):
                self.set(self._oldvalue)
        elif self._datatype == 'float':
            if float(val) < float(self.spinbox['from']):
                self.set(self._oldvalue)
        if self._datatype == 'integer':
            return int(self.spinbox.get())
        elif self._datatype == 'float':
            return float(self.spinbox.get())

    def set(self, value=None):
        return self.tk.call(self.spinbox._w, 'set', value)

    def trace(self, mode, callback):
        # this is really only useful if mode == 'w'
        self._var.trace(mode, callback)

    def disable(self):
        self.spinbox.config(state='disabled')
        if self.label:
            self.label.config(state='disabled')
        if self.label2:
            self.label2.config(state='disabled')

    def enable(self):
        self.spinbox.config(state='normal')
        if self.label:
            self.label.config(state='normal')
        if self.label2:
            self.label2.config(state='normal')

    def focus_set(self):
        self.spinbox.focus_set()
    focus = focus_set

    def focus_get(self):
        return self.spinbox.focus_get()

    def bind(self, sequence, callback, *args):
        return self.spinbox.bind(sequence, callback, *args)

    def selection(self, *args):
        return self.spinbox.selection(*args)
