# -*- coding: utf-8 -*-

import Tkinter
import ttk
from tkMessageBox import askyesno, showerror, showinfo, showwarning
import os
import sys
from subprocess import call, Popen, PIPE
from commands import getoutput, getstatusoutput
import types
import TkTray
from ScrolledWidgets import ScrolledListbox, ScrolledText
from UnicodeHandler import fsdecode, fsencode
import ConfigHandler
from Counter import Counter
from BaseDialog import BaseDialog
from Icon import Icon

#############################################################################

VERSION = u'1.3.2'
ABOUT = u'''%s
Version: %s
(c) 2015-2017 Michael Lange <klappnase@tuxfamily.org>
License: GPL v.2''' % (_('A simple software update notification applet.'), VERSION)

PROGRAMDIR = os.path.abspath(sys.path[0])
ICONDIR = os.path.join(PROGRAMDIR, 'icons')
CONFIGDIR = os.path.join(os.getenv('HOME'), '.config', 'un-applet')

UPGRADESCRIPT = 'un-applet-dist-upgrade'
UPDATESCRIPT = 'un-applet-update-pkgdatabase'
SYSTEMSETTINGS = 'un-applet-system-settings'
STATEFILE = '/var/lib/un-applet/state'
PKGSFILE = '/var/lib/un-applet/pkgnames'
LASTRUNFILE = '/var/lib/un-applet/lastrun'
READMEFILE = '/usr/share/doc/un-applet/README'
# messages to be displayed on the terminal emulator during the dist-upgrade
# or update of the package database,
# these are only here so that they appear in the .pot file
term_msgs = (_('Running command:'),
             _('Updating the list of pending updates...'),
             _('Error while trying to access mirror server(s).'),
             _('Failed to update package database.'),
             _('Upgrade finished'),
             _('Please press Enter to close the window.'),
             _('Testing mirror server(s)...'),
             _('Failed to update package database.'),
             _('Checking if the package database is locked by another process...'),
             _('yes.'),
             _('Could not get lock:'),
             _('no.'),
             _('Updating package database...'),
             _('Update of package database complete.'),
             _('updated package(s) available.'),
             _('Installing updates...'),
             _('Error'),
             _('Update finished'),
             _('You configured un-applet to use aptitude to install available updates'),
             _('however aptitude does not seem to be installed on your system.'),
             _('1. use apt-get instead of aptitude to install the available updates now.'),
             _('2. do nothing and install the available updates later.'),
             _('Please enter your choice'),
             _('Update cancelled'),
             _('Software update has been cancelled.'),
             _('Invalid input'),
             _('Generating the list of available updates...'),
             _('Starting update of the package database...'),
             _('ok'),
             _('error'),
             _('mirror server(s) could not be reached.')
             )

def which(progname):
    paths = os.getenv('PATH').split(os.pathsep)
    for p in paths:
        full = os.path.join(p, progname)
        if os.path.exists(full) and os.access(full, os.X_OK):
            return full
    return None

def get_terminal_cmd(preferred=None):
    terminal = None
    alternatives = ['mlterm', 'urxvt', 'xvt']
    if preferred:
        alternatives = [preferred] + alternatives
    for a in alternatives:
        terminal = which(a)
        if terminal:
            break
    return terminal

def lastrun():
    try:
        f = open(LASTRUNFILE, 'r')
        n = int(f.read().strip())
        f.close()
        return getoutput('date -d "@%d"' % n).strip()
    except (IOError, OSError, ValueError):
        return None

def lastrun_msg():
    lr = lastrun()
    if not lr:
        lr = 'N/A'
    return _('last query') + u': ' + fsdecode(lr)


#############################################################################

class ReadonlyCombobox(ttk.Combobox):
    # ttk.Combobox widget in readonly state with a focus highlight
    # idea borrowed from ScrolledText
    def __init__(self, master=None, **kw):
        self.frame = Tkinter.Frame(master, highlightthickness=1)
        ttk.Combobox.__init__(self, self.frame, state='readonly', **kw)
        self.pack(side='left', fill='both', expand=1)
        text_meths = vars(ttk.Combobox).keys()
        methods = vars(Tkinter.Pack).keys() + vars(Tkinter.Grid).keys() + \
                  vars(Tkinter.Place).keys()
        methods = set(methods).difference(text_meths)
        for m in methods:
            if m[0] != '_' and m != 'config' and m != 'configure':
                setattr(self, m, getattr(self.frame, m))
    def __str__(self):
        return str(self.frame)

#############################################################################

class Msgbox(BaseDialog):
    def __init__(self, message='', detail='', text='', icon='error', **kw):
        BaseDialog.__init__(self, **kw)
        self.grid_rowconfigure(2, weight=1)
        self.grid_columnconfigure(1, weight=1)

        self.img_info = Icon('dialog_info')
        self.img_error = Icon('dialog_error')
        if icon == 'info':
            self.icon = Tkinter.Label(self, image=self.img_info)
        elif icon == 'error':
            self.icon = Tkinter.Label(self, image=self.img_error)
        self.icon.grid(row=0, column=0, rowspan=2, padx=10, pady=10, sticky='w')

        self.msg = Tkinter.Label(self, font='TkCaptionFont',
                                    text=message, anchor='w')
        self.msg.grid(row=0, column=1, padx=10, pady=(10, 3), sticky='w')
        self.detail = Tkinter.Label(self, justify='left', text=detail)
        self.detail.grid(row=1, column=1, padx=10, pady=(3, 10), sticky='w')

        self.text = ScrolledText(self, width=80, height=12, wrap='word')
        self.text.grid(row=2, column=0, columnspan=2, padx=10, pady=10,
                                sticky='news')
        self.text.insert('end', text)
        self.text.configure(state='disabled')

        self.button = ttk.Button(self, text='OK', command=self.hide)
        self.button.grid(row=3, column=0, columnspan=2, pady=(0, 8))
        self.assign_hotkey(self.button)

    def hide(self, event=None):
        self.destroy()

    def show(self):
        BaseDialog.show(self)
        self.button.focus_set()
        self.grab_set()

def msgbox_showinfo(message='', detail='', text=''):
    m = Msgbox(message, detail, text, icon='info')
    m.show()

def msgbox_showerror(message='', detail='', text=''):
    m = Msgbox(message, detail, text, icon='error')
    m.show()

def show_info_terminal_failure():
    msg = _('No supported terminal emulator application found.') + u'\n' +\
          _('Please make sure that one of the programs') +\
          u' "xvt", "rxvt-unicode" ' +  _('or') + u' "mlterm" ' +\
          _('is installed on your system.')
    showerror(title='Update Notifier Applet', message=_('Error'),
                detail=msg)

#############################################################################

# a hopefully sensible selection of colors from rgb.txt which should
# hopefully work with any terminal
RGB_TXT = '''White Black Blue1 Blue2 Blue3 Blue4 Cyan1 Cyan2 Cyan3 Cyan4
 Green1 Green2 Green3 Green4 Yellow1 Yellow2 Yellow3 Yellow4 Orange1 Orange2
 Orange3 Orange4 Red1 Red2 Red3 Red4 Pink1 Pink2 Pink3 Pink4 Magenta1 Magenta2
 Magenta3 Magenta4 Purple1 Purple2 Purple3 Purple4 Tan1 Tan2 Tan3 Tan4 Brown1
 Brown2 Brown3 Brown4 Lightgreen Lightblue Lightcyan Lightyellow Darkblue
 Darkcyan Darkmagenta Darkred AntiqueWhite LightGoldenrod MistyRose
 PaleTurquoise PowderBlue DarkOliveGreen DarkSlateGray MidnightBlue Firebrick
 DarkSlateBlue Gray10 Gray20 Gray30 Gray40 Gray50 Gray60
 Gray70 Gray80 Gray90'''
COLORS = [c.strip() for c in RGB_TXT.split()]

#############################################################################

class PreferencesDialog(BaseDialog):
    def __init__(self, trayicon=None, **kw):
        BaseDialog.__init__(self, **kw)
        self._pipe = None
        self.trayicon = trayicon# reference for switching icon on theme change

        self.grid_columnconfigure(1, weight=1)

        self.pref_term = Tkinter.StringVar(value=ConfigHandler.get('PREF_TERM'))
        self.term_bg = Tkinter.StringVar(value=ConfigHandler.get('TERM_BG'))
        self.term_fg = Tkinter.StringVar(value=ConfigHandler.get('TERM_FG'))
        self.gksu_cmd = Tkinter.StringVar(value=ConfigHandler.get('GKSU_CMD'))
        self.icon_theme = Tkinter.StringVar(value=ConfigHandler.get('ICON_THEME'))

        self.img_err = Icon('error')
        self.img_prefs = Icon('preferences')
        self.icon = Tkinter.Label(self, image=self.img_prefs)
        self.icon.grid(row=0, column=0, rowspan=2, padx=10, pady=10, sticky='w')

        self.msg = Tkinter.Label(self, font='TkCaptionFont',
                                    text=_('Preferences'), anchor='w')
        self.msg.grid(row=0, column=1, padx=10, pady=(10, 3), sticky='w')
        self.detail = Tkinter.Label(self, justify='left',
                text=_('Customize the behavior of un-applet'))
        self.detail.grid(row=1, column=1, padx=10, pady=(3, 10), sticky='w')

        self.frame = Tkinter.Frame(self, relief='groove', bd=2)
        self.frame.grid(row=2, column=0, columnspan=2, padx=5, pady=5, sticky='news')
        self.frame.grid_columnconfigure(1, weight=1)

        Tkinter.Label(self.frame, text=_('Time interval after which to poll for pending updates:'), anchor='w').grid(
                        row=2, column=0, padx=(10,0), pady=(15,10), sticky='w')
        self.poll_int = Counter(self.frame,
            label2=_('minutes'), value=ConfigHandler.get('POLL_INTERVAL'),
            width=6, from_=3, to=9999)
        self.poll_int.grid(row=2, column=1, padx=10, pady=(15,10), sticky='w')

        Tkinter.Label(self.frame, text=_('Startup delay (for IceWM system tray):'), anchor='w').grid(
                        row=3, column=0, padx=(10,0), pady=(0,10), sticky='w')
        self.startup_delay = Counter(self.frame,
            label2=_('seconds'), value=ConfigHandler.get('STARTUP_DELAY'),
            width=6, from_=0, to=20)
        self.startup_delay.grid(row=3, column=1, padx=10, pady=(0,10), sticky='w')

        Tkinter.Label(self.frame, text=_('Graphical administrator login command:'), anchor='w').grid(
                        row=4, column=0, padx=(10,0), pady=(0,10), sticky='w')
        self.e2 = Tkinter.Entry(self.frame, textvariable=self.gksu_cmd)
        self.e2.grid(row=4, column=1, padx=10, pady=(0,10), sticky='we')

        Tkinter.Label(self.frame, text=_('Icon theme:'), anchor='w').grid(
                        row=5, column=0, padx=(10,0), pady=(0,10), sticky='w')
        self.cb0 = ReadonlyCombobox(self.frame, values=(), textvariable=self.icon_theme)
        self.cb0.grid(row=5, column=1, padx=10, pady=(0, 10), sticky='w')
        self.cb0.bind('<<ComboboxSelected>>', self.preview_icontheme)

        lf = Tkinter.LabelFrame(self.frame, text=_('Terminal emulator options:'))
        lf.grid(row=6, column=0, columnspan=2, padx=3, pady=0, sticky='nsew')
        lf.grid_columnconfigure(2, weight=1)

        f1 = Tkinter.Frame(lf)
        f1.grid(row=0, column=0, columnspan=2, sticky='ew')
        Tkinter.Label(f1, text=_('Preferred terminal application (if available):')).grid(
                        row=0, column=0, columnspan=3, sticky='w', padx=5, pady=(5,0))
        self.rb1 = ttk.Radiobutton(f1, text='Xvt', value='xvt', variable=self.pref_term)
        self.rb1.grid(row=1, column=0, padx=10, pady=5, sticky='w')
        self.rb2 = ttk.Radiobutton(f1, text='Rxvt-Unicode', value='urxvt', variable=self.pref_term)
        self.rb2.grid(row=1, column=1, padx=10, pady=5, sticky='w')
        self.rb3 = ttk.Radiobutton(f1, text='MLTerm', value='mlterm', variable=self.pref_term)
        self.rb3.grid(row=1, column=2, padx=10, pady=5, sticky='w')

        Tkinter.Label(lf, text=_('Background color:')).grid(
                                row=1, column=0, padx=5, pady=5, sticky='w')
        self.cb1 = ReadonlyCombobox(lf, values=COLORS, textvariable=self.term_bg)
        self.cb1.grid(row=1, column=1, padx=5, pady=5, sticky='w')
        Tkinter.Label(lf, text=_('Text color:')).grid(
                                row=2, column=0, padx=5, pady=5, sticky='w')
        self.cb2 = ReadonlyCombobox(lf, values=COLORS, textvariable=self.term_fg)
        self.cb2.grid(row=2, column=1, padx=5, pady=5, sticky='w')
        self.previewtext = 'Spam, Spam, Spam, Spam...\n' +\
                           'Lovely Spam! Wonderful Spam!'
        self.preview = Tkinter.Label(lf, relief='sunken', bd=2, image='',
            compound='left', padx=5, font='TkFixedFont', justify='left',
            anchor='w', text=self.previewtext)
        try:
            self.preview.configure(fg=self.term_fg.get(), bg=self.term_bg.get())
        except Tkinter.TclError:
            # just in case...
            self.term_fg.set('Black')
            self.term_bg.set('White')
            self.preview.configure(fg=self.term_fg.get(), bg=self.term_bg.get())
        self.preview.grid(row=1, column=2, rowspan=2, pady=5, padx=5, sticky='nsew')
        self.term_fg.trace('w', self.update_preview_fg)
        self.term_bg.trace('w', self.update_preview_bg)
        Tkinter.Label(lf, text=_('Please note:') + u' ' +\
                _('colors might be reversed on some systems/terminals.')).grid(
                row=3, column=0, columnspan=3, padx=5, pady=5, sticky='w')

        f1 = Tkinter.Frame(lf)
        f1.grid(row=4, column=0, columnspan=3, sticky='w')
        f1.grid_columnconfigure(0, weight=1)

        Tkinter.Label(f1, text=_('Window size (in characters):')).grid(
            row=0, column=0, padx=5, pady=5, sticky='w')
        self.term_width = Counter(f1, label=_('Width:'),
            value=ConfigHandler.get('TERM_WIDTH'),
            width=6, from_=40, to=400)
        self.term_width.grid(row=0, column=1, padx=5, pady=5, sticky='e')
        self.term_height = Counter(f1, label=_('Height:'),
            value=ConfigHandler.get('TERM_HEIGHT'),
            width=6, from_=10, to=150)
        self.term_height.grid(row=0, column=2, padx=5, pady=5, sticky='e')

        Tkinter.Label(f1, text=_('Window position (in pixels):')).grid(
            row=1, column=0, padx=5, pady=5, sticky='w')
        self.term_xoff = Counter(f1, label=_('Horizontal pos.:'),
            value=ConfigHandler.get('TERM_XOFF'),
            width=6, from_=0, to=1000)
        self.term_xoff.grid(row=1, column=1, padx=5, pady=5, sticky='e')
        self.term_yoff = Counter(f1, label=_('Vertical pos.:'),
            value=ConfigHandler.get('TERM_YOFF'),
            width=6, from_=0, to=1000)
        self.term_yoff.grid(row=1, column=2, padx=5, pady=5, sticky='e')

        ttk.Button(lf, text=_('Test terminal configuration...'),
                    command=self.test_terminal).grid(
                    row=5, column=0, padx=5, pady=5, sticky='w')

        buttonbox = Tkinter.Frame(self, relief='flat', bd=0)
        buttonbox.grid(row=7, column=0, columnspan=2, sticky='ew')
        buttonbox.grid_columnconfigure(0, weight=1)
        buttonbox.grid_columnconfigure(1, weight=1)
        self.applybutton = ttk.Button(buttonbox, text=_('Apply'), command=self.apply_prefs)
        self.applybutton.grid(row=0, column=0, pady=8, padx=15, sticky='e')
        self.cancelbutton = ttk.Button(buttonbox, text=_('Cancel'), command=self.hide)
        self.cancelbutton.grid(row=0, column=1, pady=8, padx=15, sticky='w')
        self.assign_hotkey(self.applybutton)
        self.assign_hotkey(self.cancelbutton)

    ##################################################################

    def check_iconthemes(self):
        iconthemes = ['Default']
        dirnames = [os.path.join(ICONDIR, d) for d in os.listdir(ICONDIR) \
                        if os.path.isdir(os.path.join(ICONDIR, d))] + \
                   [os.path.join(CONFIGDIR, d) for d in os.listdir(CONFIGDIR) \
                        if os.path.isdir(os.path.join(CONFIGDIR, d))]
        for d in dirnames:
            try:
                # check if directory contains at least one .png file
                for f in os.listdir(d):
                    if os.path.splitext(f)[1] == '.png':
                        iconthemes.append(os.path.basename(d))
                        break
            except:# read permission denied
                continue
        iconthemes.sort()
        self.cb0.configure(values=iconthemes)
        self.icon_theme.set(ConfigHandler.get('ICON_THEME'))

    def preview_icontheme(self, event):
        oldtheme = ConfigHandler.get('ICON_THEME')
        ConfigHandler.set('ICON_THEME', self.icon_theme.get())
        self.trayicon.image(self.trayicon.image())
        ConfigHandler.set('ICON_THEME', oldtheme)

    def check_installed_terminals(self):
        if not which('xvt'):
            self.rb1.configure(state='disabled')
        else:
            self.rb1.configure(state='normal')
        if not which('urxvt'):
            self.rb2.configure(state='disabled')
        else:
            self.rb2.configure(state='normal')
        if not which('mlterm'):
            self.rb3.configure(state='disabled')
        else:
            self.rb3.configure(state='normal')

    def update_preview_bg(self, *args):
        oldbg = self.preview.cget('bg')
        try:
            self.preview.configure(bg=self.term_bg.get())
        except Tkinter.TclError:
            m = _('Color not supported') + u':\n' + fsdecode(self.term_bg.get())
            self.preview.configure(text=m, image=self.img_err,
                                    bg='white', fg='black')
            self.term_bg.set(oldbg)
            def restore_preview_bg():
                self.preview.configure(text=self.previewtext, image='',
                                        fg=self.term_fg.get(), bg=oldbg)
            self.after(2000, restore_preview_bg)

    def update_preview_fg(self, *args):
        oldfg = self.preview.cget('fg')
        try:
            self.preview.configure(fg=self.term_fg.get())
        except Tkinter.TclError:
            m = _('Color not supported') + u':\n' + fsdecode(self.term_fg.get())
            self.preview.configure(text=m, image=self.img_err,
                                    bg='white', fg='black')
            self.term_fg.set(oldfg)
            def restore_preview_fg():
                self.preview.configure(text=self.previewtext, image='',
                                        bg=self.term_bg.get(), fg=oldfg)
            self.after(2000, restore_preview_fg)

    def test_terminal(self):
        if not self._pipe is None:
            msg = _('A test window is already running.')
            return showerror(title='Update Notifier Applet',
                             message=_('Error'), detail=msg)

        terminal = get_terminal_cmd(preferred=self.pref_term.get())
        if not terminal:
            return show_info_terminal_failure()
        term = '%s -title SoftwareUpdate -sb -geometry %dx%d+%d+%d -fg %s -bg %s -e' % (
                    terminal,
                    self.term_width.get(),
                    self.term_height.get(),
                    self.term_xoff.get(),
                    self.term_yoff.get(),
                    self.term_fg.get(),
                    self.term_bg.get())
        prompt1 = _('Test window for your terminal emulator configuration.')
        prompt2 = _('Please press Enter to close the window.')
        args = term.split() +\
            ['python', '-c', u'print("%s"); raw_input("%s")' % (prompt1, prompt2)]
        self._pipe = Popen(args)
        self.after(500, self.poll_test_terminal)

    def poll_test_terminal(self):
        # we have to call at least self._pipe.poll(), otherwise we end up
        # with a zombie process after the terminal is closed
        p = self._pipe.poll()
        if not p is None:
            pipe = self._pipe
            del(pipe)
            self._pipe = None
            return
        self.after(500, self.poll_test_terminal)

    ##################################################################

    def validate_prefs(self):
        ok = True

        try:
            gk = self.gksu_cmd.get().strip()
            if not gk:
                ok = False
                msg = _('Please define a root login program to use.')
            gk = gk.split()[0]
            if os.path.isfile(gk):
                if not os.access(gk, os.X_OK):
                    ok = False
                    msg = _('Unable to execute given command:') + u'\n' +\
                          _('Permission denied.')
            elif not which(gk):
                ok = False
                msg = _('Program not found:') + u'\n' + fsdecode(gk)
        except IndexError:
            ok = False
            msg = _('Please define a root login program to use.')
        if not ok:
            showerror(title='Update Notifier Applet', message=_('Error'),
                         detail=msg)
            self.e2.focus_set()
            self.e2.selection_clear()
            self.e2.selection_range(0, 'end')
            return False

        for w in (self.poll_int, self.startup_delay, self.term_width,
                    self.term_height, self.term_xoff, self.term_yoff):
            try:
                # don't forget to convert w.spinbox['from'/'to'] into a float
                # first, since Tk will return strings that look like floats!
                min, max, value = (int(float(w.spinbox['from'])),
                                    int(float(w.spinbox['to'])), int(w.get()))
                if not min <= value <= max:
                    ok = False
            except ValueError:
                ok = False
            if not ok:
                msg = _('Please choose an integer value between %d and %d') % (min, max)
                showerror(title='Update Notifier Applet', message=_('Error'),
                             detail=msg)
                w.focus_set()
                w.selection('clear')
                w.selection('range', 0, 'end')
                return False

        if self.term_bg.get() == self.term_fg.get():
            msg = _('Please choose different fore- and background colors.')
            showerror(title='Update Notifier Applet', message=_('Error'),
                         detail=msg)
            return False

        return True

    def apply_prefs(self):
        if self.validate_prefs():
            ConfigHandler.set('POLL_INTERVAL', self.poll_int.get())
            ConfigHandler.set('STARTUP_DELAY', self.startup_delay.get())
            ConfigHandler.set('GKSU_CMD', self.gksu_cmd.get().strip())
            ConfigHandler.set('TERM_BG', self.term_bg.get())
            ConfigHandler.set('TERM_FG', self.term_fg.get())
            ConfigHandler.set('TERM_WIDTH', self.term_width.get())
            ConfigHandler.set('TERM_HEIGHT', self.term_height.get())
            ConfigHandler.set('TERM_XOFF', self.term_xoff.get())
            ConfigHandler.set('TERM_YOFF', self.term_yoff.get())
            ConfigHandler.set('PREF_TERM', self.pref_term.get())
            ConfigHandler.set('ICON_THEME', self.icon_theme.get())
            ConfigHandler.save()
            self.hide()

    def hide(self):
        for w in (self.poll_int, self.startup_delay, self.term_width,
                    self.term_height, self.term_xoff, self.term_yoff):
            w.selection('clear')
        for w in (self.e2, self.cb1, self.cb2):
            w.selection_clear()
        if not self._pipe is None:
            self._pipe.terminate()
        self.trayicon.image(self.trayicon.image())
        BaseDialog.hide(self)

    def show(self):
        self.poll_int.set(ConfigHandler.get('POLL_INTERVAL'))
        self.startup_delay.set(ConfigHandler.get('STARTUP_DELAY'))
        self.gksu_cmd.set(ConfigHandler.get('GKSU_CMD'))
        self.term_bg.set(ConfigHandler.get('TERM_BG'))
        self.term_fg.set(ConfigHandler.get('TERM_FG'))
        self.term_width.set(ConfigHandler.get('TERM_WIDTH'))
        self.term_height.set(ConfigHandler.get('TERM_HEIGHT'))
        self.term_xoff.set(ConfigHandler.get('TERM_XOFF'))
        self.term_yoff.set(ConfigHandler.get('TERM_YOFF'))
        self.pref_term.set(ConfigHandler.get('PREF_TERM'))
        self.check_installed_terminals()
        self.check_iconthemes()
        BaseDialog.show(self)

#############################################################################

class PkgListViewer(BaseDialog):
    def __init__(self, *args, **kw):

        self.comand = None
        if 'command' in kw:
            self.command = kw['command']
            del(kw['command'])
        BaseDialog.__init__(self, *args, **kw)

        self.grid_rowconfigure(2, weight=1)
        self.grid_columnconfigure(1, weight=1)

        self.img_info = Icon('dialog_info')
        self.img_error = Icon('dialog_error')
        self.icon = Tkinter.Label(self, image=self.img_info)
        self.icon.grid(row=0, column=0, rowspan=2, padx=10, pady=10, sticky='w')

        self.msg = Tkinter.Label(self, font='TkCaptionFont',
                                    text=_('Updates available'), anchor='w')
        self.msg.grid(row=0, column=1, padx=10, pady=(10, 3), sticky='w')
        self.detail = Tkinter.Label(self, justify='left',
                text=_('Updates for the following packages are available:'))
        self.detail.grid(row=1, column=1, padx=10, pady=(3, 10), sticky='w')

        self.slistbox = ScrolledListbox(self, width=150, height=200)
        self.slistbox.grid(row=2, column=0, columnspan=2, padx=10, pady=10,
                                sticky='news')
        self.listbox = self.slistbox.listbox
        self.listbox.configure(selectmode='single')

        self.msg_verinfo = Tkinter.Label(self, justify='left',
                text=_('Version information for package "%s":') % u'N/A')
        self.msg_verinfo.grid(row=3, column=0, columnspan=2,
                                padx=10, pady=3, sticky='w')

        self.verinfo = Tkinter.Label(self, text='', relief='sunken', bd=1,
                                justify='left', bg='white', font='TkFixedFont',
                                padx=6, pady=6, anchor='w')
        self.verinfo.grid(row=4, column=0, columnspan=2,
                                padx=10, pady=10, sticky='news')

        buttonbox = Tkinter.Frame(self)
        buttonbox.grid(row=5, column=0, columnspan=2, sticky='ew')
        buttonbox.grid_columnconfigure(0, weight=1)
        buttonbox.grid_columnconfigure(1, weight=1)
        self.okbutton = ttk.Button(buttonbox, text='OK', command=self.on_ok)
        self.okbutton.grid(row=0, column=0,  pady=(0, 8), sticky='e', padx=15)
        self.cancelbutton = ttk.Button(buttonbox, text=_('Cancel'), command=self.hide)
        self.cancelbutton.grid(row=0, column=1, pady=(0, 8), sticky='w', padx=15)

        self.assign_hotkey(self.okbutton)
        self.assign_hotkey(self.cancelbutton)
        self.listbox.bind('<<ListboxSelect>>', self.show_pkginfo)

    def hide(self, event=None):
        BaseDialog.hide(self)

    def on_ok(self):
        self.hide()
        self.command()

    def show(self):
        self.update_pkglist()
        BaseDialog.show(self)
        self.okbutton.focus_set()

    def show_pkginfo(self, event=None):
        sel = self.listbox.curselection()
        if not sel:
            self.msg_verinfo.configure(
                    text=_('Version information for package "%s":') % u'N/A')
            self.verinfo.configure(text='N/A')
            return
        pkgname = self.listbox.get(sel[0])
        self.msg_verinfo.configure(
            text=_('Version information for package "%s":') % fsdecode(pkgname))
        sts, info = getstatusoutput('apt-cache policy %s' % fsencode(pkgname))
        if sts or (not info):
            info = 'N/A'
        self.verinfo.configure(text=info)

    def update_pkglist(self):
        self.listbox.select_clear(0, 'end')
        self.listbox.delete(0, 'end')
        try:
            f = open(PKGSFILE, 'r')
            lines = f.readlines()
            f.close()
            pkgs = [x.strip() for x in lines if x.strip()]
            self.icon.configure(image=self.img_info)
            self.msg.configure(
                text=_('Install pending update(s) now?'))
            detail = _('Updates for the following %d package(s) are available') % len(pkgs) +\
                     u'\n(' + lastrun_msg() + u'):'
            self.detail.configure(text=detail)

            for p in pkgs:
                self.listbox.insert('end', p.strip())
            if pkgs:
                self.listbox.select_set(0)
                self.show_pkginfo()
        except (IOError, OSError):
            pkgs = []
            self.icon.configure(image=self.img_error)
            self.msg.configure(text=_('Error'))
            self.detail.configure(
                text=_('Error while trying to read %s') % fsdecode(PKGSFILE))

#############################################################################

class UpdateNotifierApplet(TkTray.Icon):
    def __init__(self, master, **kw):
        self.img = Icon('updates-ok')
        TkTray.Icon.__init__(self, master, image=self.img,
                             tooltip=_('Software Update'), docked=0, **kw)
        # hack for Icewm system tray:
        self.update_idletasks()
        self.configure(docked=1)
        self.update_idletasks()

        self._id = None
        self._state = 0
        self._pipe = None
        self._poll_lock = 0
        self._backround_proc = 0# 1=update 2=dist-upgrade 3=system-settings

        self.preferences_dialog = PreferencesDialog(trayicon=self.img)
        self.pkglist_viewer = PkgListViewer(command=self.dist_upgrade)

        self.menu.add_command(label=_('Start upgrade...'),
                                command=self.dist_upgrade)
        self.menu.add_command(label=_('Update package database...'),
                                command=self.update_pkg_database)
        self.menu.add_command(label=_('System-wide settings...'),
                                command=self.system_settings)
        self.menu.add_command(label=_('Show available upgrades...'),
                                command=self.show_state)
        self.menu.add_command(label=_('Preferences...'),
                                command=self.show_pref_dialog)
        self.menu.add_command(label=_('About...'),
                                command=self.show_about)
        self.menu.add_separator()
        self.menu.add_command(label=_('Quit'), command=self.close)
        self.bind('<1>', self.show_state)

        if not os.path.exists(STATEFILE):
            self.firstrun()
        else:
            self.poll_loop()

    def firstrun(self):
        msg = _('Maybe you are running the applet for the first time?') + u'\n'
        msg += _('Do you want to search for pending updates now?')
        res = askyesno(title='Update Notifier Applet',
                            message=_('Status file not found'),
                            detail=msg, default='yes')
        if res:
            self.update_pkg_database(firstrun=True)
        else:
            self.poll_loop()

    def get_state(self):
        try:
            f = open(STATEFILE, 'r')
            n = int(f.read().strip())
            f.close()
            return n
        except (IOError, OSError, ValueError):
            return -256

    def poll(self):
        while self._poll_lock:
            pass
        self._poll_lock = 1
        self._state = self.get_state()
        if self._state > 0:
            newimg = 'updates-available'
        elif self._state < 0:
            newimg = 'updates-error'
        else:
            newimg = 'updates-ok'
        if newimg != self.img.image():
            self.img.image(newimg)
        self._poll_lock = 0

    def poll_loop(self):
        self.poll()
        self._id = self.after(60000*ConfigHandler.get('POLL_INTERVAL'), self.poll_loop)

    ###########################################################################

    def dist_upgrade(self):
        if self._state == 0:
            msg = _('There do not seem to be any updates available.') + u'\n' +\
                  _('Do you want to try to install pending updates anyway?')
            if not askyesno(title='Update Notifier Applet', message=_('Really?'),
                            detail=msg, default='no'):
                return
        elif self._state == -1:
            return self.show_info_mirror_failure()
        elif self._state == -2:
            return self.show_info_lock_failure()
        elif self._state <= -3:
            return self.show_info_statefile_failure()

        if not self._pipe is None:
            return self.show_info_background_process()

        term_cmd = self.terminal_cmd()
        if not term_cmd:
            return show_info_terminal_failure()

        self._background_proc = 2
        # gksu might be configured with arguments, e.g. -w to
        # force su-mode, so use split() on the GKSU_CMD.
        # The command line must be passed as a single string to
        # Popen, otherwise gksu will fail to work
        gksu_cmd = ConfigHandler.get('GKSU_CMD').split()
        args = gksu_cmd + ['%s -e %s' % (term_cmd, UPGRADESCRIPT)]
        self._pipe = Popen(args, stdout=PIPE, stderr=PIPE, shell=False)
        if not self._id is None:
            _id = self._id
            self._id = None
            self.after_cancel(_id)
        self.after(500, self.poll_upgrade)

    def system_settings(self):
        if not self._pipe is None:
            return self.show_info_background_process()
        self._background_proc = 3
        gksu_cmd = ConfigHandler.get('GKSU_CMD').split()
        args = gksu_cmd + [SYSTEMSETTINGS]
        self._pipe = Popen(args, stdout=PIPE, stderr=PIPE, shell=False)
        if not self._id is None:
            _id = self._id
            self._id = None
            self.after_cancel(_id)
        self.after(500, self.poll_upgrade)

    def update_pkg_database(self, firstrun=False):
        if firstrun:
            res = True
        else:
            msg = _('Do you want to search for pending updates now?')
            res = askyesno(title='Update Notifier Applet',
                            message=_('Please confirm'),
                            detail=msg, default='yes')
        if res:
            if not self._pipe is None:
                return self.show_info_background_process()

            term_cmd = self.terminal_cmd()
            if not term_cmd:
                return show_info_terminal_failure()

            self._background_proc = 1
            gksu_cmd = ConfigHandler.get('GKSU_CMD').split()
            args = gksu_cmd + ['%s -e %s -i' % (term_cmd, UPDATESCRIPT)]
            self._pipe = Popen(args, stdout=PIPE, stderr=PIPE, shell=False)
            if not self._id is None:
                _id = self._id
                self._id = None
                self.after_cancel(_id)
            self.after(500, self.poll_upgrade)

    def terminal_cmd(self):
        terminal = get_terminal_cmd(preferred=ConfigHandler.get('PREF_TERM'))
        if not terminal:
            return None

        term_cmd = '%s -title SoftwareUpdate -sb ' % terminal
        if os.path.isfile(os.path.join(ICONDIR, 'un-applet.xpm')):
            if os.path.basename(terminal) == 'urxvt':
                term_cmd += '-icon %s ' % os.path.join(ICONDIR, 'un-applet.xpm')
            elif os.path.basename(terminal) == 'mlterm':
                term_cmd += '--iconpath %s ' % os.path.join(ICONDIR, 'un-applet.xpm')

        term_cmd += '-geometry %dx%d+%d+%d -fg %s -bg %s' % (
                        ConfigHandler.get('TERM_WIDTH'),
                        ConfigHandler.get('TERM_HEIGHT'),
                        ConfigHandler.get('TERM_XOFF'),
                        ConfigHandler.get('TERM_YOFF'),
                        ConfigHandler.get('TERM_FG'),
                        ConfigHandler.get('TERM_BG'))
        return term_cmd

    ###########################################################################

    def poll_upgrade(self):
        # we have to call at least self._pipe.poll(), otherwise we end up
        # with a zombie process after the terminal is closed
        p = self._pipe.poll()
        if not p is None:
            pipe = self._pipe
            del(pipe)
            self._pipe = None
            self._background_proc = 0
            self.poll_loop()
            return
        self.after(500, self.poll_upgrade)

    def show_about(self):
        try:
            f = open(READMEFILE, 'r')
            msg = f.read()
            f.close()
        except (IOError, OSError):
            msg = _('Error while trying to open "%s".') % READMEFILE
        msgbox_showinfo(message=_('About UN-Applet'), detail=ABOUT, text=msg)

    def show_info_background_process(self):
        bp = u'.\n'
        if self._background_proc == 1:
            bp = u'\n(%s).\n' % _('update of the package database')
        elif self._background_proc == 2:
            bp = u'\n(%s).\n' % _('software upgrade')
        elif self._background_proc == 3:
            bp = u'\n(%s).\n' % _('un-applet System Settings')
        msg = _('It seems like a background process is already running') +\
                  bp + _('Please try again later.')
        showerror(title='Update Notifier Applet', message=_('Error'),
                         detail=msg)

    def show_info_lock_failure(self):
        msg = _('Error while trying to get lock /var/lib/dpkg/lock') + u' ' +\
              _('or /var/lib/apt/lists/lock.') + u'\n\n' +\
              _('Possible reason:') + u'\n' + \
              _('The package database was locked by another process') + u' ' +\
              _('while trying to check for pending updates.') + u'\n\n' +\
              _('Possible solutions:') + u'\n\n' +\
              _('Make sure that no other package management application is running') + u', ' +\
              _('then run "Update package database..." from the menu') + u' ' +\
              _('or simply wait until the next automatic query for pending updates is performed.')

        msgbox_showerror(message=_('Error'),
                    detail=_('Please note the error details below:'), text=msg)

    def show_info_mirror_failure(self):
        msg = _('Error while trying to download package data files.') + u'\n' +\
              _('One or more mirror servers could not be reached.') + u'\n\n' +\
              _('Possible reasons:') + u'\n' + \
              _('1. your network connection might be down or unreliable') + u'\n' +\
              _('2. one or more of the mirrors defined in %s or %s ') % (u'/etc/apt/sources.list', u'/etc/apt/sources.list.d/*.list') +\
              _('is temporarily or permanently not working.') + u'\n\n' +\
              _('In case 2. you can try to update the package database at a later time') + u' ' +\
              _('or, if the problem persists, remove the broken mirror server(s) from the sources.list file(s).') + u'\n\n' + \
              _('If you want to skip the test of the mirror servers') + u' ' +\
              _('before updating the package database, you may open the file /etc/un-applet/un-applet.conf') + u' ' +\
              _('and set VERIFY_MIRRORS="0" .') + u'\n' +\
              _('However, due to a bug in apt-get this may cause packages erroneously selected for upgrade') + u' ' +\
              _('if one or more mirrors are unreachable, especially if you use apt-pinning') + u' ' +\
              _('which may very well be the case even if you never consciously decided to do so.') + u'\n\n' +\
              _('You can obtain more detailed information about which mirrors are not reachable') + u' ' +\
              _('by running "Update package database..." from the context menu and examining the output messages.') + u' ' +\
              _('It is possible that this will even fix the problem') + u' ' +\
              _('in case that e.g. a temporary network connection issue has been resolved.')

        msgbox_showerror(message=_('Error'),
                    detail=_('Please note the error details below:'), text=msg)

    def show_info_statefile_failure(self):
        msg = _('Error while trying to analyse the file /var/lib/un-applet/state') + u'\n\n' +\
              _('Possible reasons:') + u'\n' + \
              _('1. the file has not yet been created because') + u' ' +\
              _('you are running the Update Notifier Applet for the first time.') + u'\n' +\
              _('2. the file was removed or modified by another process.') + u'\n\n' +\
              _('Possible solutions:') + u'\n\n' +\
              _('You can run "Update package database..." from the menu') + u' ' +\
              _('or simply wait until the next automatic query for pending updates is performed.')

        msgbox_showerror(message=_('Error'),
                    detail=_('Please note the error details below:'), text=msg)

    def show_info_uptodate(self):
        msg = _('No updates available, your system is up to date.') + \
                        u'\n(' + lastrun_msg() + u')'
        showinfo(title='Update Notifier Applet', message=_('Information'),
                        detail=msg)

    def show_state(self, event=None):
        self.poll()
        if self._state == 0:
            self.show_info_uptodate()
        elif self._state > 0:
            self.pkglist_viewer.show()
        elif self._state == -1:
            self.show_info_mirror_failure()
        elif self._state == -2:
            self.show_info_lock_failure()
        elif self._state <= -3:
            self.show_info_statefile_failure()

    def show_pref_dialog(self):
        self.preferences_dialog.show()

    def close(self):
        if not self._id is None:
            _id = self._id
            self._id = None
            self.after_cancel(_id)
        self.preferences_dialog.hide()
        self.preferences_dialog.destroy()
        self.pkglist_viewer.destroy()
        self.destroy()
        self.master.remove(self)
